from ..base import KNOWN_OBJECTS, EventIOObject
from .simtelfile import SimTelFile
from .objects import (
    ADCSamples,
    ADCSums,
    ArrayEvent,
    AuxiliaryAnalogTraces,
    AuxiliaryDigitalTraces,
    CalibrationEvent,
    CameraMonitoring,
    CameraOrganization,
    CameraSettings,
    CameraSoftwareSettings,
    DisabledPixels,
    DriveSettings,
    FSPhot,
    History,
    HistoryCommandLine,
    HistoryConfig,
    HistoryMeta,
    ImageParameters,
    LaserCalibration,
    MCEvent,
    MCPhotoelectronSum,
    MCRunHeader,
    MCRunStatistics,
    MCShower,
    PixelCalibration,
    PixelList,
    PixelMonitoring,
    PixelSettings,
    PixelTiming,
    PixelTriggerTimes,
    PointingCorrection,
    RunHeader,
    RunStatistics,
    StereoReconstruction,
    TelescopeEvent,
    TelescopeEventHeader,
    TelescopeObject,
    TrackingPosition,
    TriggerInformation,
)

__all__ = [
    'ADCSamples',
    'ADCSums',
    'ArrayEvent',
    'AuxiliaryAnalogTraces',
    'AuxiliaryDigitalTraces',
    'CalibrationEvent',
    'CameraMonitoring',
    'CameraOrganization',
    'CameraSettings',
    'CameraSoftwareSettings',
    'DisabledPixels',
    'DriveSettings',
    'FSPhot',
    'History',
    'HistoryCommandLine',
    'HistoryConfig',
    'HistoryMeta',
    'ImageParameters',
    'LaserCalibration',
    'MCEvent',
    'MCPhotoelectronSum',
    'MCRunHeader',
    'MCRunStatistics',
    'MCShower',
    'PixelCalibration',
    'PixelList',
    'PixelMonitoring',
    'PixelSettings',
    'PixelTiming',
    'PixelTriggerTimes',
    'PointingCorrection',
    'RunHeader',
    'RunStatistics',
    'SimTelFile',
    'StereoReconstruction',
    'TelescopeEvent',
    'TelescopeEventHeader',
    'TrackingPosition',
    'TriggerInformation',
]

for cls in EventIOObject.__subclasses__():
    KNOWN_OBJECTS[cls.eventio_type] = cls

for cls in TelescopeObject.__subclasses__():
    KNOWN_OBJECTS[cls.eventio_type] = cls

for tel_id in range(1000):
    KNOWN_OBJECTS[TelescopeEvent.telid_to_type(tel_id)] = TelescopeEvent

for tel_id in range(1000):
    KNOWN_OBJECTS[TrackingPosition.telid_to_type(tel_id)] = TrackingPosition



def assert_exact_version(self, supported_version):
    if self.header.version != supported_version:
        raise NotImplementedError(
            (
                'Unsupported version of {name}:'
                ' only supports version {supported_version}'
                ', got {given_version}'
            ).format(
                name=self.__class__.__name__,
                supported_version=supported_version,
                given_version=self.header.version,
            )
        )


def assert_version_in(self, supported_versions):
    if self.header.version not in supported_versions:
        raise NotImplementedError(
            (
                'Unsupported version of {name}:'
                ' supported versions are: {supported_versions},'
                ' got: {given_version} '
            ).format(
                name=self.__class__.__name__,
                supported_versions=supported_versions,
                given_version=self.header.version,
            )
        )


def assert_max_version(self, last_supported):
    if self.header.version > last_supported:
        raise NotImplementedError(
            (
                'Unsupported version of {name}:'
                ' only versions up to {last_supported} supported,'
                ' got: {given_version} '
            ).format(
                name=self.__class__.__name__,
                last_supported=last_supported,
                given_version=self.header.version,
            )
        )



from eventio import EventIOFile
from argparse import ArgumentParser
from eventio.simtel import HistoryMeta
from eventio.search_utils import yield_toplevel_of_type

parser = ArgumentParser()
parser.add_argument('inputfile')
parser.add_argument('--encoding', default='utf8', help='Encoding to use for decoding METAPARAMs')


def main():
    args = parser.parse_args()

    with EventIOFile(args.inputfile) as f:
        found_meta = False

        for o in f:
            # METAPARAM objects come at the toplevel after the history blocks
            # read all history meta and as soon as another objects comes, stop
            # so we don't have to read through the complete file
            if isinstance(o, HistoryMeta):
                found_meta = True
            else:
                if found_meta:
                    break
                else:
                    continue

            if o.header.id == -1:
                s = "Global METAPARAMs"
                print()
                print(s)
                print(len(s) * "-")
            else:
                s = f"METAPARAMs for telescope={o.header.id}"
                print()
                print(s)
                print(len(s) * "-")

            for k, v in o.parse().items():
                print(k.decode(args.encoding), "=", v.decode(args.encoding))



if __name__ == '__main__':
    main()

